"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * @file index
 * @author Cuttle Cong
 * @date 2018/3/26
 * @description
 */
const pReduce = require("p-reduce");
const eval_1 = require("../../lib/eval");
const cli_1 = require("./cli");
const _ = require("lodash");
const debug = require('debug')('edam:promptProcessor');
function prompt(prompts, { yes = true, promptProcess = cli_1.default, context = {} } = {}) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!_.isFunction(promptProcess)) {
            throw new Error('prompt is missing the process');
        }
        debug('input: %O', prompts);
        const res = yield pReduce(prompts, function (set, prompt) {
            return __awaiter(this, void 0, void 0, function* () {
                prompt = Object.assign({}, prompt);
                let allow = true;
                if (_.isString(prompt.when)) {
                    allow = !!eval_1.default(prompt.when, set);
                }
                else if (_.isFunction(prompt.when)) {
                    allow = yield prompt.when(set);
                }
                if (!allow)
                    return set;
                // ${git.email}
                // ${git.name}
                // ${pm}
                // ${dirName}
                if (_.isString(prompt.default)) {
                    const replaceCtx = _.merge({}, set, context);
                    prompt.default = prompt.default.replace(/\${(.+?)}/g, (__, key) => {
                        if (_.hasIn(replaceCtx, key)) {
                            return _.get(replaceCtx, key).toString();
                        }
                        return __;
                    });
                }
                if (_.isFunction(prompt.default)) {
                    prompt.default = yield prompt.default(set, context);
                }
                let value;
                const { transformer, validate } = prompt;
                delete prompt.when;
                delete prompt.transformer;
                delete prompt.validate;
                if (yes && prompt.yes !== false) {
                    value = prompt.default;
                }
                else {
                    value = yield promptProcess(prompt);
                }
                if (_.isFunction(transformer)) {
                    value = yield transformer(value, set, context);
                }
                if (_.isFunction(validate)) {
                    let message = yield validate(value, set, context);
                    if (typeof message === 'string') {
                        value = prompt.default;
                        this.logger &&
                            this.logger.warn('validate %s failed, error message: %s, so the value "%s" fallback to default value.', prompt.name, message, value);
                        return set;
                    }
                }
                Object.assign(set, {
                    [prompt.name]: value
                });
                return set;
            });
        }, {});
        debug('output: %O', res);
        return res;
    });
}
exports.default = prompt;
